import os
import sys


import readinputs


def epiAndSampleToFSCpar(params, fpaths):

    population=params["population"]
    sampleSize=params["sampleSize"]
    bottleneck=params["bottleneck"]
    recombination=params["recombination"]
    genomeSize=params["genomeSize"]
    mutation=params["mutation"]
    mutationBias=params["mutationBias"]
    replicates=params["nReplicates"]
    
    for iRep in range(replicates):
        print("Converting epi and samples data to fsc26 parameters for replicate " + str(iRep))
        repDir=fpaths['replicateDirBase'] + str(iRep) + fpaths['filepathSeparator']
        if not os.path.isdir(repDir):
            os.mkdir(repDir)
        epiDataFpath=repDir + fpaths['epiDataLeafName']
        inputSampleFpath=repDir + fpaths['sampleDataLeafName']
        EpiAndSample = readinputs.readEpiAndSample(epiDataFpath, inputSampleFpath)
        Hosts = EpiAndSample["Hosts"]
        Starts = EpiAndSample["Starts"]
        Infectors = EpiAndSample["Infectors"]
        SamplingDates = EpiAndSample["SamplingDates"]
        lastSamplingDate = EpiAndSample["lastSamplingDate"]
        
        fscParFPath=repDir + fpaths['fastSimCoalParametersLeafName']
        oFile=open(fscParFPath,"w")
        oFile.write("//Number of population samples (demes)\n")
        oFile.write(str(len(Hosts))+" samples to simulate :\n")
        oFile.write("//Population effective sizes (number of genes)\n")
        for h in Hosts:
            oFile.write(str(population)+"\n")
        oFile.write("//Sample sizes\n")
        for n in range(len(Hosts)):
            oFile.write(str(sampleSize)+" "+str(lastSamplingDate-SamplingDates[n])+"\n")
        oFile.write("//Growth rates	: negative growth implies population expansion\n")
        for h in Hosts:
            oFile.write("0\n")
        oFile.write("//Number of migration matrices : 0 implies no migration between demes\n")
        oFile.write("0\n")
        oFile.write("//historical event: time, source, sink, migrants, new size, new growth rate, migr. matrix \n")
        oFile.write(str((2*len(Hosts))-1)+"  historical event \n")
        for n in range(len(Hosts)):
            oFile.write(str(lastSamplingDate-Starts[n])+" "+str(n)+" "+str(n)+" 0 "+str(float(bottleneck)/population)+" 0 0 \n")
            if Infectors[n]!=0:
                oFile.write(str(lastSamplingDate-Starts[n]+1)+" "+str(n)+" "+str(Hosts.index(Infectors[n]))+" 1 1 0 0 \n")
        oFile.write("//Number of independent loci [chromosome] \n")
        oFile.write("1 0\n")
        oFile.write("//Per chromosome: Number of linkage blocks\n")
        oFile.write("1\n")
        oFile.write("//per Block: data type, num loci, rec. rate and mut rate + optional parameters\n")
        oFile.write("DNA "+str(genomeSize)+" "+str(recombination)+" "+str(mutation)+" "+str(mutationBias)+"\n")
        oFile.close()
            

simfpathsFpath = ''
assert len(sys.argv) == 2
simfpathsFpath = sys.argv[1]
assert len(simfpathsFpath) > 0


fpaths = readinputs.readFilePaths(simfpathsFpath)
params = readinputs.readParameters(fpaths)
epiAndSampleToFSCpar(params, fpaths)

