import random
import sys

import readinputs



def readSequences(simOutPutarpFpath, nHosts):
    with open(simOutPutarpFpath) as file:
        line=file.readline()
        linelist=line.split()
        while len(linelist)<2 or linelist[0]!="SampleName=\"Sample":
            line=file.readline()
            linelist=line.split()
            if line=="":
                print("empty file "+simOutPutarpFpath)
                exit()
        polymDNASeqs=[[] for i in range(nHosts)]
        for n in range(nHosts):
            line=file.readline()
            line=file.readline()
            line=file.readline()
            linelist=line.split()
            host=int(linelist[0].split("_")[0])
            assert host-1 == n
            for g in range(params['sampleSize']):
                polymDNASeqs[n].append(linelist[2])
                line=file.readline()
                linelist=line.split()
            line=file.readline()
            line=file.readline()
        file.close()
    return polymDNASeqs


def repDirectory(iRep):
    return fpaths['replicateDirBase'] + str(iRep) + fpaths['filepathSeparator']



def makeSampledSubset(iRep):
    ## Read input files
    repDir = repDirectory(iRep)
    epiDataFpath = repDir + fpaths['epiDataLeafName']
    inputSampleFpath = repDir + fpaths['sampleDataLeafName']
    EpiAndSample = readinputs.readEpiAndSample(epiDataFpath, inputSampleFpath)
    nHosts = len(EpiAndSample['Hosts'])
    SampleIsActual = EpiAndSample['SampleIsActual']
    
    nCasesSampled = sum(SampleIsActual)

    #Read simulation output
    fsc26parName = fpaths['fastSimCoalParametersLeafName']
    tokens = fsc26parName.split('.', maxsplit=1)
    assert len(tokens) == 2
    assert tokens[1] == 'par'
    fsc26Name = tokens[0]

    simOutPutarpFpath = repDir + fsc26Name + fpaths['filepathSeparator'] + fsc26Name + "_1_1.arp"
    allPolymDNASeqs = readSequences(simOutPutarpFpath, nHosts)
    hss = 0
    hostSubset = [-1 for i in range(nCasesSampled)]
    SampleNamesSubset = [-1 for i in range(nCasesSampled)]
    SamplingDatesSubset = [-1 for i in range(nCasesSampled)]
    polymDNASeqsSubset = [[] for i in range(nCasesSampled)]
    for h in range(nHosts):
        if SampleIsActual[h] == 1:
            hostSubset[hss] = EpiAndSample['Hosts'][h]
            SampleNamesSubset[hss] = EpiAndSample['SampleNames'][h]
            SamplingDatesSubset[hss] = EpiAndSample['SamplingDates'][h] / params['UNITSPERDAY']
            polymDNASeqsSubset[hss] = allPolymDNASeqs[h]
            hss += 1
    sampleCountsSubset = [ [] for i in range(nCasesSampled) ]
    DNA=["A","C","G","T"]
    for host in range(nCasesSampled):
        assert len(polymDNASeqsSubset[host]) == params['sampleSize']
        for site in range(len(polymDNASeqsSubset[host][0])):
            counts = [0,0,0,0]
            for seq in range(len(polymDNASeqsSubset[host])):
                nucleotide = polymDNASeqsSubset[host][seq][site]
                if params['errRate'] == 0.0:
                    counts[DNA.index(nucleotide)] += 1
                else:
                    if random.random()>=params['errRate']:
                       counts[DNA.index(nucleotide)] += 1
                    else:
                       counts[(DNA.index(nucleotide) + random.randint(1,3))%4] += 1
            counttxt = str(counts[0])+"-"+str(counts[1])+"-"+str(counts[2])+"-"+str(counts[3])+", "
            if (site % 10) == 9 or site == len(polymDNASeqsSubset[host][0]) - 1:
                counttxt = counttxt + "\n"
            sampleCountsSubset[host].append(counttxt)
    # Errors for non-variable sites
    LengGenome = params['genomeSize']
    nSNPs=len(sampleCountsSubset[0])
    nNonVs = int( (LengGenome-nSNPs)/4 )
    for host in range(nCasesSampled):
        site = 0
        for k in range(nNonVs):
            for k2 in range(4):
                counts=[0,0,0,0]
                for s in range(params['sampleSize']):
                    if params['errRate']==0.0:
                        counts[k2]+=1
                    else:
                        if random.random()>=params['errRate']:
                            counts[k2]+=1
                        else:
                           counts[(k2+random.randint(1,3))%4]+=1
                counttxt = str(counts[0])+"-"+str(counts[1])+"-"+str(counts[2])+"-"+str(counts[3])
                if site < nNonVs * 4:
                    counttxt = counttxt+", "
                    if (site % 10) == 9:
                        counttxt = counttxt + "\n"
                sampleCountsSubset[host].append(counttxt)
                site = site + 1


    for i in range(len(sampleCountsSubset)):
        sampleCountsSubset[i]="".join(sampleCountsSubset[i])
    
    sampledSubset = {
        'nCasesSampled' : nCasesSampled,
        'SampleNamesSubset' : SampleNamesSubset,
        'SamplingDatesSubset' : SamplingDatesSubset,
        'hostSubset' : hostSubset,
        'sampleCountsSubset' : sampleCountsSubset
        }
    return sampledSubset




def writeBEASTXML(iRep, sampledSubset, unobservedCases):
    virtualN = params['virtualPop']
    Nmcmc = params['mcmc']
    theta = params['estimateTheta']
    sampleEvery = params['sampleEvery']

    nCasesSampled = sampledSubset['nCasesSampled']
    SampleNamesSubset = sampledSubset['SampleNamesSubset']
    SamplingDatesSubset = sampledSubset['SamplingDatesSubset']
    hostSubset = sampledSubset['hostSubset']
    sampleCountsSubset = sampledSubset['sampleCountsSubset']
    
    BEASTXMLfpath = repDirectory(iRep) + fpaths['BeastXMLLeafName']
    if unobservedCases:
        BEASTXMLfpath = BEASTXMLfpath + '-uo.xml'
    else:
        BEASTXMLfpath = BEASTXMLfpath + '-std.xml'

    startBeforeSequencing = params["startDaysBeforeSeq"]
    endAfterSequencing = params["endDaysAfterSeq"]
    
    with open(BEASTXMLfpath,"w") as outF:
        outF.write("<beast version='2.0'\n namespace=\'PoMo:beast.util:beast.core:beast.evolution:beast.math:beast.evolution.alignment:beast.core.parameter:beast.core.util:beast.evolution.likelihood:beast.evolution.sitemodel:beast.evolution.substitutionmodel:beast.math.distributions:beast.evolution.speciation:beast.evolution.tree:beast.evolution.operators\'>\n\n\n")
        
        outF.write("<alignmentPoMo spec=\"beast.evolution.alignment.AlignmentPoMo\" id=\"alignment\" dataTypePoMo=\"PoModata\">\n\n")
        for n in range(nCasesSampled):
            outF.write("    <sequence spec=\"SequencePoMo\" id=\"s"+str(SampleNamesSubset[n])+"\" taxon=\"t"+str(SampleNamesSubset[n])+"\" value=\"" +sampleCountsSubset[n]+"\"/>\n\n")
        outF.write("</alignmentPoMo>\n\n\n")
        traitsetOpener = "    <TraitSetPoMo spec=\'beast.evolution.tree.TraitSetPoMo\' id=\'traitSet\' direction=\"forward\"   units=\"day\""
        if unobservedCases:
            outF.write(traitsetOpener + " unobservedCases=\"true\"\n")
        else:
            outF.write(traitsetOpener + "\n")
        outF.write("        samplesHostsValue=\"")
        for n in range(nCasesSampled):
            suffix = ", " if (n < nCasesSampled-1) else "\"\n"
            outF.write("t"+str(SampleNamesSubset[n])+"=H"+str(hostSubset[n]) + suffix)
            
        outF.write("        samplesDatesValue=\"")
        for n in range(nCasesSampled):
            suffix = ", " if (n < nCasesSampled-1) else "\"\n"
            outF.write("t"+str(SampleNamesSubset[n])+"="+str(round(SamplingDatesSubset[n], 2)) + suffix)
            
        outF.write("        HostDatesStartValue=\"")
        for n in range(nCasesSampled):
            suffix = ", " if (n < nCasesSampled-1) else "\"\n"
            outF.write("H"+str(hostSubset[n])+"="+str(round(SamplingDatesSubset[n] - startBeforeSequencing, 2)) + suffix)
        
        outF.write("        HostDatesEndValue=\"")
        for n in range(nCasesSampled):
            suffix = ", " if (n < nCasesSampled-1) else "\">\n"
            outF.write("H"+str(hostSubset[n])+"="+str(round(SamplingDatesSubset[n] + endAfterSequencing, 2)) + suffix)
    
        outF.write("      <taxa spec=\'beast.evolution.alignment.TaxonSet\' alignment=\'@alignment\'/>\n   </TraitSetPoMo>\n\n\n")
        
        outF.write("    <treePoMo spec=\'beast.evolution.tree.TreePoMo\' id=\'tree\' estimate=\'true\'>\n        <traits spec=\'beast.evolution.tree.TraitSetPoMo\' idref=\'traitSet\'/>\n    </treePoMo>\n\n\n")
    
    
        outF.write("<!-- Substitution model (PoMo + HKY), theta (if used) is the frequency of polymorphisms at the root.\n")
        outF.write("     If root nuc freqs are not estimated, the ones from mutation rates are used. -->\n")
        outF.write("  <siteModel spec=\"SiteModelPoMo\" id=\"siteModel\">\n")
        outF.write("    <mutationRate spec=\'RealParameter\' id=\"mutationRate\" value=\"0.001\" lower=\"0.0\"/>\n")
        outF.write("    <substModel spec=\"PoMoGeneral\" virtualPop=\""+str(virtualN)+"\" estimateRootFreqs=\"false\" useTheta=\""+str(theta)+"\" theta=\"0.01\" id=\"PoMo.substModel\">\n")
        outF.write("      <fitness spec=\'RealParameter\' id=\"PoMo.fitness\" value=\"1.0 1.0 1.0 1.0\" lower=\"0.8\" upper=\"1.2\"/>\n")
        outF.write("      <rootNucFreqs spec=\'RealParameter\' id=\"PoMo.rootFreqs\" value=\"0.25 0.25 0.25 0.25\"/>\n")
        outF.write("      <mutModel spec=\'MutationModel\' id=\"PoMo.mutModel\">\n")
        outF.write("        <rateVector spec=\'RealParameter\' id=\"PoMo.mutRates\" value=\"0.01 0.01\" upper=\"0.3\"/>\n")
        outF.write("        <nucFreqs spec=\'RealParameter\' id=\"PoMo.nucFreqs\" value=\"0.25 0.25 0.25 0.25\"/>\n")
        outF.write("      </mutModel>\n")
        outF.write("    </substModel>\n")
        outF.write("  </siteModel>\n\n\n")
    
        
        outF.write("  <input spec=\'CompoundDistribution\' id=\'parameterPriors\'>\n")
        outF.write("    <distribution spec=\'beast.math.distributions.Prior\' x=\"@PoMo.mutRates\">\n")
        outF.write("      <distr spec=\'LogNormalDistributionModel\' M=\"-7.0\" S=\"2.0\"/>\n")
        outF.write("    </distribution>\n")
        outF.write("    <distribution spec=\'beast.math.distributions.Prior\' x=\"@mutationRate\">\n")
        outF.write("      <distr spec=\'LogNormalDistributionModel\' M=\"-7.0\" S=\"2.0\"/>\n")
        outF.write("    </distribution>\n  ")
        outF.write("    <distribution spec=\'beast.math.distributions.Prior\' x=\"@PoMo.bottleneck\">\n")
        outF.write("      <distr spec=\'LogNormalDistributionModel\' M=\"6.0\" S=\"3.0\"/>\n")
        outF.write("    </distribution>\n")
        outF.write("  </input>\n\n\n")
    
        outF.write("  <input spec='TreePoMoCoVPrior' id=\"treePrior\" tree=\"@tree\">\n")
        outF.write("  </input>\n\n")
    
        seqErrVal="0.0"
        seqErrOperatorText=""
        if params['errRate']>0.0000001:
            seqErrVal="0.0001"
            seqErrOperatorText="    <operator spec=\'ScaleOperator\' id=\'errScaler\'\n           parameter=\"@PoMo.seqError\"\n           scaleFactor=\"0.95\" weight=\"5\"/>\n\n"
    
        outF.write("   <input spec=\'PoMoCoVTreeLikelihood\' id=\"treeLikelihood\" treePoMo=\"@tree\">\n")
        outF.write("     <seqError spec=\'RealParameter\' id=\"PoMo.seqError\" value=\""+seqErrVal+"\" upper=\"0.01\"/>\n")
        outF.write("     <bottleneck spec=\'RealParameter\' id=\"PoMo.bottleneck\" value=\"1.0\" upper=\"10000\"/>\n")
        outF.write("     <data idref=\"alignment\"/>\n")
        outF.write("     <siteModel idref=\'siteModel\'/>\n")
        outF.write("   </input>\n\n\n")
        
        outF.write("   <run spec=\"MCMC\" id=\"mcmc\" chainLength=\""+str(Nmcmc)+"\" storeEvery=\"50000\">\n")
        outF.write("     <state>\n       <stateNode idref=\"tree\"/>\n")
        outF.write("       <stateNode idref=\"PoMo.bottleneck\"/>\n")
        outF.write("       <stateNode idref=\"mutationRate\"/>\n")
        outF.write("       <stateNode idref=\"PoMo.fitness\"/>\n")
        outF.write("       <stateNode idref=\"PoMo.mutRates\"/>\n")
        outF.write("       <stateNode idref=\"PoMo.seqError\"/>\n")
        outF.write("       <stateNode idref=\"PoMo.nucFreqs\"/>\n")
        outF.write("       <stateNode idref=\"PoMo.rootFreqs\"/>\n")
        outF.write("     </state>\n\n")
        outF.write("     <distribution spec=\'CompoundDistribution\' id=\'posterior\'>\n")
        outF.write("     <distribution idref=\"treeLikelihood\"/>\n")
        outF.write("     <distribution idref=\"treePrior\"/>\n")
        outF.write("     <distribution idref=\"parameterPriors\"/>\n")
        outF.write("     </distribution>\n\n\n")  #       <stateNode idref=\'birthRate\'/>\n
    
    
        outF.write("    <operator spec=\'ScaleOperator\' id=\'mutScalerAll\' parameter=\"@PoMo.mutRates\" scaleFactor=\"0.95\" weight=\"5\" scaleAll=\"true\"/>\n\n")
        outF.write("    <operator spec=\'ScaleOperator\' id=\'mutScalerOne\' parameter=\"@PoMo.mutRates\" scaleFactor=\"0.98\" weight=\"5\" scaleAll=\"false\"/>\n\n")
        outF.write(seqErrOperatorText)
        outF.write("    <operator spec=\'ScaleOperator\' id=\'muRateScaler\' parameter=\"@mutationRate\" scaleFactor=\"0.95\" weight=\"5\"/>\n\n")
        outF.write("    <operator spec=\'ScaleOperator\' id=\'bottleneckScaler\' parameter=\"@PoMo.bottleneck\" scaleFactor=\"0.9\" weight=\"5\"/>\n\n")
        outF.write("    <operator spec=\'UpDownOperator\' id=\'RelativeScalerMuHKY\' scaleFactor=\"0.95\" weight=\"5\" down=\"@mutationRate\" up=\"@PoMo.mutRates\"/>\n\n\n")

        outF.write("    <operator spec=\'UniformTransmission\' weight=\"50\" tree=\"@tree\"/>\n\n")
        outF.write("    <operator spec=\'WilsonBaldingTransmission\' weight=\"50\" tree=\"@tree\"/>\n\n")
        outF.write("    <operator spec=\'ParentChildExchange\' weight=\"50\" tree=\"@tree\"/>\n\n")
        outF.write("    <operator spec=\'PoMoCoVCountsWalker\' weight=\"50\" tree=\"@tree\"/>\n\n")

    
        #GRJ I changed this to just use the leaf names PoMo_transmission.log, PoMo_transmission.trees
        # If I recall correctly:
        # There is a command line option (-working is the relevant one, I think)
        # Used by double-clicking beast.exe, -working is set, and leaf name should be used
        # From command line, can choose behaviour, but I think using -working is best.
        # .log file
        logfname = 'transmission-uo.log' if unobservedCases else 'transmission-std.log'
        outF.write("    <logger logEvery=\"" + str(sampleEvery) + "\" fileName=\"" + logfname + "\">\n")
        outF.write("      <model idref=\'posterior\'/>\n")
        outF.write("      <log idref=\"posterior\"/>\n")
        outF.write("      <log idref=\"parameterPriors\"/>\n")
        outF.write("      <log idref=\"treePrior\"/>\n")
        outF.write("      <log idref=\"treeLikelihood\"/>\n")
        outF.write("      <log idref=\"mutationRate\"/>\n")
        outF.write("      <log idref=\"PoMo.bottleneck\"/>\n")
        outF.write("      <log idref=\"PoMo.mutRates\"/>\n")
        outF.write("      <log idref=\"PoMo.seqError\"/>\n")
        outF.write("      <log idref=\"PoMo.nucFreqs\"/>\n")
        outF.write("    </logger>\n\n")
        
        # .trees file
        treefname = 'transmission-uo.trees' if unobservedCases else 'transmission-std.trees'
        outF.write("    <logger logEvery=\"" + str(sampleEvery) + "\" fileName=\"" + treefname + "\" mode=\"tree\">\n")
        outF.write("      <log idref=\"tree\"/>\n")
        outF.write("    </logger>\n\n")
        
        # to screen
        outF.write("    <logger logEvery=\""+str(sampleEvery)+"\">\n")
        outF.write("      <model idref=\'posterior\'/>\n")
        outF.write("      <log idref=\"posterior\"/>\n")
        outF.write("      <log idref=\"parameterPriors\"/>\n")
        outF.write("      <log idref=\"treePrior\"/>\n")
        outF.write("      <log idref=\"treeLikelihood\"/>\n")
        outF.write("      <log idref=\"mutationRate\"/>\n")
        outF.write("      <log idref=\"PoMo.bottleneck\"/>\n")
        outF.write("      <log idref=\"PoMo.seqError\"/>\n")
        outF.write("      <ESS spec=\'ESS\' name=\'log\' arg=\"@posterior\"/>\n")
        outF.write("      <ESS spec=\'ESS\' name=\'log\' arg=\"@treeLikelihood\"/>\n")
        outF.write("    </logger>\n\n")
        
        outF.write("</run>\n </beast>\n")
    
        outF.close()



###############################################################################
###############################################################################
###############################################################################

simfpathsFpath = ''
assert len(sys.argv) == 2
simfpathsFpath = sys.argv[1]
assert len(simfpathsFpath) > 0

fpaths = readinputs.readFilePaths(simfpathsFpath)
params = readinputs.readParameters(fpaths)
replicates = params['nReplicates']

for iRep in range(replicates):
    print('Making XML for replicate ' + str(iRep))
    sampledSubset = makeSampledSubset(iRep)
    writeBEASTXML(iRep, sampledSubset, True)
    writeBEASTXML(iRep, sampledSubset, False)



    
    
    