


def readValue(f, name):
    line = f.readline()
    line = line.strip()
    tokens = line.split(" ", maxsplit=1)
    assert len(tokens) == 2
    assert tokens[0] == name
    return tokens[1]   


def readIntValue(f, name):
    val = readValue(f, name)
    return int(val)
    
    
def readFloatValue(f, name):
    val = readValue(f, name)
    return float(val)


def readFilePaths(SimulationFpathsFpath):
    with open(SimulationFpathsFpath) as f:
        val = readValue(f, 'name')
        assert val == 'value'
        filepathSeparator = readValue(f, 'filepathSeparator')
        epiDataLeafName = readValue(f, 'epiDataLeafName')
        sampleDataLeafName = readValue(f, 'sampleDataLeafName')
        fastSimCoalParametersLeafName = readValue(f, 'fastSimCoalParametersLeafName')
        BeastXMLLeafName = 	readValue(f, 'BeastXMLLeafName')
        paramsFpath = readValue(f, 'paramsFpath')
        replicateDirBase = readValue(f, 'replicateDirBase')
        runBatchFPath = readValue(f, 'runBatchFPath')
        fsc26FPath = readValue(f, 'fsc26FPath')
    fpaths = {
            "filepathSeparator" : filepathSeparator,
            "epiDataLeafName" : epiDataLeafName,
            "sampleDataLeafName" : sampleDataLeafName,
            "fastSimCoalParametersLeafName" : fastSimCoalParametersLeafName,
            "BeastXMLLeafName" : BeastXMLLeafName,
            "paramsFpath" : paramsFpath,
            "replicateDirBase" : replicateDirBase,
            "runBatchFPath" : runBatchFPath,
            "fsc26FPath" : fsc26FPath,
            }
    return fpaths


def readParameters(fpaths):
    paramsFPath = fpaths["paramsFpath"]
    with open(paramsFPath) as f:
        val = readValue(f, 'name')
        assert val == 'value'
        nReplicates = readIntValue(f, 'nReplicates')
        nSusceptibles = readIntValue(f, 'nSusceptibles')
        minHosts = readIntValue(f, 'minHosts')
        minSamples = readIntValue(f, 'minSamples')
        PrHostIsSampled = readFloatValue(f, 'PrHostIsSampled')
        UNITSPERDAY = readFloatValue(f, 'UNITSPERDAY')
        infMaxDays = readFloatValue(f, 'infMaxDays')
        startDaysBeforeSeq = readFloatValue(f, 'startDaysBeforeSeq')
        endDaysAfterSeq = readFloatValue(f, 'endDaysAfterSeq')
        infShape = readFloatValue(f, 'infShape')
        infScale = readFloatValue(f, 'infScale')
        infIntensity = readFloatValue(f, 'infIntensity')
        population = readIntValue(f, 'population')
        bottleneck = readIntValue(f, 'bottleneck')
        recombination = readFloatValue(f, 'recombination')
        mutation = readFloatValue(f, 'mutation')
        mutationBias = readFloatValue(f, 'mutationBias')
        genomeSize = readIntValue(f, 'genomeSize')
        errRate = readFloatValue(f, 'errRate')
        sampleSize = readIntValue(f, 'sampleSize')
        virtualPop = readIntValue(f, 'virtualPop')
        sampleEvery = readIntValue(f, 'sampleEvery')
        mcmc = readIntValue(f, 'mcmc')
        estimateTheta = readIntValue(f, 'estimateTheta') is not 0
    params = { 
        "nReplicates" : nReplicates,
        "nSusceptibles" : nSusceptibles,
        "minHosts" : minHosts,
        "minSamples" : minSamples,
        "PrHostIsSampled" : PrHostIsSampled,
        "UNITSPERDAY" : UNITSPERDAY,
        "infMaxDays" : infMaxDays,
        "startDaysBeforeSeq" : startDaysBeforeSeq,
        "endDaysAfterSeq" : endDaysAfterSeq,
        "infShape" : infShape,
        "infScale" : infScale,
        "infIntensity" : infIntensity,
        "population" : population,
        "bottleneck" : bottleneck,
        "recombination" : recombination,
        "mutation" : mutation,
        "mutationBias" : mutationBias,
        "genomeSize" : genomeSize,
        "errRate" : errRate,
        "sampleSize" : sampleSize,
        "virtualPop" : virtualPop,
        "sampleEvery" : sampleEvery,
        "mcmc" : mcmc,
        "estimateTheta" : estimateTheta
              }
    return params


def readEpiAndSample(epiDataFpath, inputSampleFpath):
    ## Read input files
    Hosts=[]
    Starts=[]
    Ends=[]
    Infectors=[]
    SampleNames=[]
    SamplingDates=[]
    SampleSeqs=[]
    SampleIsActual=[]
    inpF=open(epiDataFpath)
    line=inpF.readline()
    linelist=line.split()
    assert linelist[0] == 'hostID'
    assert linelist[1] == 'inf.t' 
    assert linelist[2] == 'rec.t'
    assert linelist[3] == 'infectorID'
    line=inpF.readline()
    lastSamplingDate=0
    lastTime=-100000
    while line!="" and line!="\n":
        linelist=line.split()
        Hosts.append(int(linelist[0]))
        Starts.append(int(linelist[1]))
        Ends.append(int(linelist[2]))
        if int(linelist[2])>lastTime:
            lastTime=int(linelist[2])
        Infectors.append(int(linelist[3]))
        SampleNames.append(0)
        SamplingDates.append(0)
        SampleSeqs.append([])
        SampleIsActual.append(0)
        line=inpF.readline()
    inpF.close()

    inpF=open(inputSampleFpath)
    line=inpF.readline()
    linelist=line.split()
    assert linelist[0] == 'hostID'
    assert linelist[1] == 'genomeID' 
    assert linelist[2] == 'smp.t'
    assert linelist[3] == 'actual'
    line=inpF.readline()
    while line!="" and line!="\n":
        linelist=line.split()
        hostIndex = Hosts.index(int(linelist[0]))
        SampleNames[hostIndex]=int(linelist[1])
        SamplingDates[hostIndex]=int(linelist[2])
        SampleIsActual[hostIndex]=int(linelist[3])
        if int(linelist[2])>lastSamplingDate:
            lastSamplingDate=int(linelist[2])
        line=inpF.readline()
    inpF.close()
    
    EpiAndSample = { 
            "Hosts" : Hosts,
            "Starts" : Starts,
            "Ends" : Ends,
            "Infectors" : Infectors,
            "SampleNames" : SampleNames,
            "SamplingDates" : SamplingDates,
            "SampleSeqs" : SampleSeqs,
            "SampleIsActual" : SampleIsActual,
            "lastSamplingDate" : lastSamplingDate
            }
    return EpiAndSample


